@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2008 Giuseppe Torelli - <colossus73@gmail.com>
+ *  Copyright (C) 2016 Ingo Brückl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -23,6 +24,7 @@
 #include "gzip_et_al.h"
 #include "interface.h"
 #include "main.h"
+#include "parser.h"
 #include "string_utils.h"
 #include "support.h"
 #include "window.h"
@@ -270,11 +272,12 @@ static gchar *xa_7zip_password_str (XArchive *archive)
 static void xa_7zip_parse_output (gchar *line, XArchive *archive)
 {
 	XEntry *entry;
-	gchar *filename;
 	gpointer item[5];
-	gint linesize = 0,a = 0;
+	gchar *filename;
 	gboolean dir;
 
+	USE_PARSER;
+
 	if (last_line)
 		return;
 
@@ -286,61 +289,62 @@ static void xa_7zip_parse_output (gchar *line, XArchive *archive)
 			archive->has_password = TRUE;
 		}
 
-		if (strncmp(line, "-----", 5) == 0)
+		if (strncmp(line, "------------------- ", 20) == 0)
 		{
 			data_line = TRUE;
 			return;
 		}
+
 		return;
 	}
+
 	if (line[0] == '-')
 	{
 		last_line = TRUE;
 		return;
 	}
 
-	linesize = strlen(line);
+	if (line[0] != ' ')
+	{
+		/* date */
+		NEXT_ITEM(item[2]);
+		/* time */
+		NEXT_ITEM(item[3]);
+	}
+	else /* invalid date/time */
+	{
+		item[2] = NULL;
+		item[3] = NULL;
+	}
 
-	/* Date */
-	line[10] = '\0';
-	item[2] = line;
-
-	/* Time */
-	line[19] = '\0';
-	item[3] = line + 11;
-
-	/* Permissions */
-	line[25] = '\0';
-	item[4] = line + 20;
+	/* attributes */
+	NEXT_ITEM(item[4]);
 
 	dir = (*(char *) item[4] == 'D');
 
-	/* Size */
-	for(a=26; a < linesize; ++a)
-		if(line[a] >= '0' && line[a] <= '9')
-			break;
+	/* size */
+	NEXT_ITEM(item[0]);
 
-	line[38] = '\0';
-	item[0] = line + a;
+	/* compressed */
 
-	/* Compressed */
-	/* Is this item solid? */
-	if (line[50] == ' ')
+	LINE_PEEK(11);
+
+	if (line[peek] == ' ')
 	{
-		line[linesize-1] = '\0';
+		line += peek + 1;
+
+		if (*line)
+			*line++ = 0;
+
 		item[1] = "0";
 	}
 	else
-	{
-		for(a=39; a < linesize; ++a)
-			if(line[a] >= '0' && line[a] <= '9')
-				break;
-		line[51] = '\0';
-		item[1] = line + a;
-		line[linesize-1] = '\0';
-	}
+		NEXT_ITEM(item[1]);
+
+	/* name (follows with two characters spacing instead of one) */
+	LAST_ITEM(filename);
+	filename++;            // skip the additional spacing character
 
-	filename = g_strdup(line + 53);
 	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
 
 	if (entry)
@@ -355,8 +359,6 @@ static void xa_7zip_parse_output (gchar *line, XArchive *archive)
 
 		archive->files_size += g_ascii_strtoull(item[0], NULL, 0);
 	}
-
-	g_free(filename);
 }
 
 void xa_7zip_list (XArchive *archive)
