@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2008 Giuseppe Torelli - <colossus73@gmail.com>
+ *  Copyright (C) 2017 Ingo Brückl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -208,8 +209,9 @@ static void xa_gzip_et_al_parse_output (gchar *line, XArchive *archive)
 
 	if (archive->type == XARCHIVETYPE_GZIP)
 	{
+		LINE_PEEK(9);
 		/* heading? */
-		if (line[9] == 'c' || line[0] == 'c')   // genuine gzip or pigz
+		if (line[peek] == 'c' || line[0] == 'c')   // genuine gzip or pigz
 			return;
 	}
 	else if (archive->type == XARCHIVETYPE_LZIP)
@@ -225,8 +227,9 @@ static void xa_gzip_et_al_parse_output (gchar *line, XArchive *archive)
 	}
 	else if (archive->type == XARCHIVETYPE_LZOP)
 	{
+		LINE_PEEK(12);
 		/* heading? */
-		if (line[12] == 'c')
+		if (line[peek] == 'c')
 			return;
 		else
 			/* method */
@@ -235,9 +238,9 @@ static void xa_gzip_et_al_parse_output (gchar *line, XArchive *archive)
 	else if (archive->type == XARCHIVETYPE_XZ)
 	{
 		/* heading? */
-		if (*line == 't')
+		if (line[0] == 't')
 			return;
-		else if (*line == 'n')
+		else if (line[0] == 'n')
 		{
 			/* "name" */
 			SKIP_ITEM;
@@ -329,7 +332,7 @@ static void xa_gzip_et_al_parse_lrzip (gchar *line, XArchive *archive)
 			first_line = FALSE;
 
 			/* pre-v0.650 */
-			if (*line != '\n')
+			if (line[0] != '\n')
 			{
 				gchar *colon = g_strrstr(line, ":\n");
 
@@ -340,7 +343,7 @@ static void xa_gzip_et_al_parse_lrzip (gchar *line, XArchive *archive)
 		}
 
 		/* since v0.650 */
-		IF_ITEM_LINE("File:")
+		IF_ITEM_LINE("File: ")
 		{
 filename:
 			LAST_ITEM(filename);
@@ -412,8 +415,8 @@ filename:
 static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 {
 	static gchar *zstdframes;
-	gchar *skipframes;
 	XEntry *entry;
+	gchar *skipframes;
 	char *pos;
 
 	USE_PARSER;
@@ -422,13 +425,13 @@ static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 		return;
 
 	if (first_line)
-		/* post-v1.3.2? (no header any more) */
+		/* post-v1.3.2 (no header any more) */
 		data_line = (strncmp(line, "===========================================", 43) != 0);
 
 	if (!data_line)
 	{
 		first_line = FALSE;
-		/* pre-v1.3.2? (end of header) */
+		/* pre-v1.3.2 (end of header) */
 		data_line = (strncmp(line, "Number of files listed:", 23) == 0);
 		return;
 	}
@@ -505,7 +508,7 @@ static void xa_gzip_et_al_parse_zstd (gchar *line, XArchive *archive)
 	g_free(zstdframes);
 }
 
-static void xa_gzip_et_al_globally_stored_entry (gchar *line, XArchive *archive)
+static void xa_gzip_et_al_prepared_items (gchar *line, XArchive *archive)
 {
 	XEntry *entry;
 	gchar *filename;
@@ -751,7 +754,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 			if (archive->type == XARCHIVETYPE_LRZIP)
 			{
 				if (archive->has_password)
-					/* no further information will be available then */
+					/* lrzip won't list content then, even if password has been given */
 					break;
 
 				first_line = TRUE;
@@ -852,7 +855,7 @@ void xa_gzip_et_al_list (XArchive *archive)
 
 		/* trigger pseudo-parser once */
 		command = g_strdup("sh -c echo");
-		archive->parse_output = xa_gzip_et_al_globally_stored_entry;
+		archive->parse_output = xa_gzip_et_al_prepared_items;
 	}
 
 	g_free(workfile);
