@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2008 Giuseppe Torelli - <colossus73@gmail.com>
+ *  Copyright (C) 2016 Ingo Brückl
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -121,8 +122,8 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 {
 	XEntry *entry;
 	gpointer item[6];
-	gboolean dir;
-	gchar *filename, *link;
+	gchar *filename;
+	gboolean dir, link;
 
 	USE_PARSER;
 
@@ -130,6 +131,7 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 	NEXT_ITEM(item[4]);
 
 	dir = (*(char *) item[4] == 'd');
+	link = (*(char *) item[4] == 'l');
 
 	/* owner/group */
 	NEXT_ITEM(item[5]);
@@ -146,23 +148,24 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 	/* name */
 	LAST_ITEM(filename);
 
-	/* symbolic link */
-
-	link = g_strrstr(filename, " -> ");
+	item[0] = NULL;
 
 	if (link)
 	{
-		*link = 0;
-		item[0] = link + 4;
+		gchar *lnk = g_strrstr(filename, " -> ");
+
+		if (lnk)
+		{
+			item[0] = lnk + 4;
+			*lnk = 0;
+		}
 	}
-	else
-		item[0] = NULL;
 
 	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
 
 	if (entry)
 	{
-		entry->is_encrypted = (archive->password != NULL);
+		entry->is_encrypted = archive->has_password;
 
 		if (dir)
 			 entry->is_dir = TRUE;
